"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Source: https://github.com/madrobby/keymaster/blob/3b1f2afabf1569848dea8b697ac418f19b601a30/keymaster.js
 */
exports.KEY_MAP = {
    "⇧": "shift",
    "⌥": "alt",
    option: "alt",
    "⌃": "control",
    ctrl: "control",
    "⌘": "meta",
    cmd: "meta",
    command: "meta",
    caps: "capslock",
    tab: "tab",
    return: "enter",
    esc: "escape",
    space: " ",
    left: "arrowleft",
    up: "arrowup",
    right: "arrowright",
    down: "arrowdown"
};
/**
 * Valid modifier keys.
 */
exports.MODIFIERS = ["shift", "control", "alt", "meta"];
/**
 * Continue propagating the event to older listeners.
 */
exports.SHOULD_PROPAGATE = true;
/**
 * Stringify a keyboard event.
 */
function keyboardEventCombo(e) {
    var keys = new Set();
    if (e.shiftKey)
        keys.add("shift");
    if (e.ctrlKey)
        keys.add("control");
    if (e.altKey)
        keys.add("alt");
    if (e.metaKey)
        keys.add("meta");
    if (exports.MODIFIERS.indexOf(e.key) === -1)
        keys.add(e.key.toLowerCase());
    return Array.from(keys)
        .sort()
        .join(" ");
}
exports.keyboardEventCombo = keyboardEventCombo;
/**
 * Map keys to string.
 */
function stringifyKey() {
    var keys = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        keys[_i] = arguments[_i];
    }
    return keys
        .map(function (key) { return String(key).toLowerCase(); })
        .map(function (key) { return (exports.KEY_MAP.hasOwnProperty(key) ? exports.KEY_MAP[key] : key); })
        .sort()
        .join(" ");
}
exports.stringifyKey = stringifyKey;
/**
 * Create a listener function from shortcuts.
 */
function createShortcuts(shortcuts, returnValue) {
    if (returnValue === void 0) { returnValue = exports.SHOULD_PROPAGATE; }
    return function (event, combo) {
        return shortcuts[combo] ? shortcuts[combo](event, combo) : returnValue;
    };
}
exports.createShortcuts = createShortcuts;
/**
 * Check if a keyboard event originated from an input.
 */
function isInputEvent(event) {
    var target = event.target;
    return (target.tagName === "INPUT" ||
        target.tagName === "SELECT" ||
        target.tagName === "TEXTAREA" ||
        target.isContentEditable);
}
exports.isInputEvent = isInputEvent;
/**
 * Function to wrap listener by filtering input events.
 */
function filterInputEvent(listener) {
    return function (event, combo) {
        return isInputEvent(event) ? exports.SHOULD_PROPAGATE : listener(event, combo);
    };
}
exports.filterInputEvent = filterInputEvent;
/**
 * Keyboard manager library for mapping key events.
 */
var Keyboard = /** @class */ (function () {
    function Keyboard() {
        this.listeners = [];
    }
    Keyboard.prototype.addListener = function (callback) {
        return this.listeners.push(callback);
    };
    Keyboard.prototype.removeListener = function (callback) {
        var indexOf = this.listeners.indexOf(callback);
        if (indexOf > -1)
            this.listeners.splice(indexOf, 1);
        return indexOf;
    };
    Keyboard.prototype.getHandler = function () {
        var listener = this.getListener();
        return function (event) {
            return listener(event, keyboardEventCombo(event));
        };
    };
    Keyboard.prototype.getListener = function (returnValue) {
        var _this = this;
        if (returnValue === void 0) { returnValue = exports.SHOULD_PROPAGATE; }
        return function (event, combo) {
            var len = _this.listeners.length;
            while (len--) {
                if (_this.listeners[len](event, combo) !== exports.SHOULD_PROPAGATE)
                    return;
            }
            return returnValue;
        };
    };
    return Keyboard;
}());
exports.Keyboard = Keyboard;
//# sourceMappingURL=index.js.map